<?php
class S2Base_ZfException extends Exception {
}

abstract class S2Base_ZfAbstractDispatcher extends Zend_Controller_Dispatcher_Standard {
    abstract protected function instantiateController(Zend_Controller_Request_Abstract $request, $moduleName, $controllerClassName);
    public function dispatch(Zend_Controller_Request_Abstract $request, Zend_Controller_Response_Abstract $response)
    {
        $this->setResponse($response);
        if (!$this->isDispatchable($request)) {
            if (!$this->getParam('useDefaultControllerAlways')) {
                require_once 'Zend/Controller/Dispatcher/Exception.php';
                throw new Zend_Controller_Dispatcher_Exception('Invalid controller specified (' . $request->getControllerName() . ')');
            }
            $className = $this->getDefaultControllerClass($request);
        } else {
            $className = $this->getControllerClass($request);
            if (!$className) {
                $className = $this->getDefaultControllerClass($request);
            }
        }
        $className = $this->loadClass($className);
        $controller = $this->instantiateController($request,
                                                   $request->getModuleName(),
                                                   $className);
        $action = $this->getActionMethod($request);
        $request->setDispatched(true);
        $disableOb = $this->getParam('disableOutputBuffering');
        $obLevel   = ob_get_level();
        if (empty($disableOb)) {
            ob_start();
        }
        try {
            $controller->dispatch($action);
        } catch (Exception $e) {
            $curObLevel = ob_get_level();
            if ($curObLevel > $obLevel) {
                do {
                    ob_get_clean();
                    $curObLevel = ob_get_level();
                } while ($curObLevel > $obLevel);
            }
            throw $e;
        }
        if (empty($disableOb)) {
            $content = ob_get_clean();
            $response->appendBody($content);
        }
        $controller = null;
    }
}

class S2Base_ZfDispatcherImpl extends S2Base_ZfAbstractDispatcher {
    const PARAM_MAX_LEN = 50;
    public function formatName($unformatted, $isAction = false) {
        return $this->_formatName($unformatted, $isAction);
    }
    protected function instantiateController(Zend_Controller_Request_Abstract $request, $moduleName, $controllerClassName) {
        $this->validateModule($request->getModuleName());
        $this->validateController($request->getControllerName());
        $this->validateAction($request->getActionName());
        $controller = $this->getControllerFromS2Container($request, $moduleName, $controllerClassName);
        if (!$this->isValidController($controller)) {
            throw new Zend_Controller_Dispatcher_Exception("Controller '$controllerClassName' is not an instance of Zend_Controller_Action");
        }
        return $controller;
    }
    protected function getControllerFromS2Container($request, $moduleName, $controllerClassName){
        $controllerName = $request->getControllerName();
        $actionName = $request->getActionName();
        $formatedActionName = $this->formatName($actionName);
        $actionMethodName = $this->getActionMethod($request);
        $controllerDir = "/app/modules/$moduleName/$controllerName";
        $actionDicon   = S2BASE_PHP5_ROOT . $controllerDir . "/dicon/$actionMethodName.dicon";
        $moduleIncFile = S2BASE_PHP5_ROOT . $controllerDir . "/$controllerName.inc.php";
        $actionIncFile = S2BASE_PHP5_ROOT . $controllerDir . "/$actionMethodName.inc.php";
        require_once($moduleIncFile);
        if (is_file($actionIncFile)) {
            require_once($actionIncFile);
        }
        $container = null;
        $cd = null;
        if (is_file($actionDicon)) {
            $container = S2ContainerFactory::create($actionDicon);
            $cd = $container->getComponentDef($controllerClassName);
        } else {
            $container = new S2ContainerImpl();
            $cd = new S2Container_ComponentDefImpl($controllerClassName);
            $container->register($cd);
        }
        if ($cd->getArgDefSize() == 0) {
            $cd->addArgDef(new S2Container_ArgDefImpl($request));
            $cd->addArgDef(new S2Container_ArgDefImpl($this->getResponse()));
            $cd->addArgDef(new S2Container_ArgDefImpl($this->getParams()));
        }
        return $container->getComponent($controllerClassName);
    }
    private function isValidController($controller){
        if ($controller instanceof Zend_Controller_Action) {
            return true;
        }
        if ($controller instanceof S2Container_DefaultAopProxy and
            $controller->target_ instanceof Zend_Controller_Action) {
            return true;
        }
        if (preg_match('/'. S2Container_AopProxyGenerator::CLASS_NAME_POSTFIX . '$/', get_class($controller)) and
            $controller->target_ instanceof Zend_Controller_Action) {
            return true;
        }
        return false;
    }
    private function validateModule($value) {
        if (!preg_match('/^[_a-zA-Z0-9]{1,' . self::PARAM_MAX_LEN .'}$/', $value)) {
            throw new S2Base_ZfException("invalid module. [$value]");
        }
    }
    private function validateController($value) {
        if (!preg_match('/^[a-zA-Z0-9]{1,' . self::PARAM_MAX_LEN .'}$/', $value)) {
            throw new S2Base_ZfException("invalid controller. [$value]");
        }
    }
    private function validateAction($value) {
        if (!preg_match('/^[_a-zA-Z0-9\.\-]{1,' . self::PARAM_MAX_LEN .'}$/', $value)) {
            throw new S2Base_ZfException("invalid action. [$value]");
        }
    }
}

interface S2Base_ZfValidateFactory {
    public function getId();
    public function getInstance($paramName, Zend_Config $config);
}

class S2Base_ZfRegexValidateFactory implements S2Base_ZfValidateFactory {
    const ID = 'regex';
    private $instance = null;
    private $validateClassName = 'Zend_Validate_Regex';
    public function getId() {
        return self::ID;
    }
    public function getInstance($paramName, Zend_Config $config) {
        $valKey = self::ID;
        if ($config->$valKey === null or $config->$valKey->pattern === null) {
            throw new S2Base_ZfException("pattern not found in Regex validation [param : $paramName]");
        }
        if ($this->instance === null) {
            Zend_Loader::loadClass($this->validateClassName);
            $this->instance = new $this->validateClassName($config->$valKey->pattern);
        } else {
            $this->instance->setPattern($config->$valKey->pattern);
        }
        return $this->instance;
    }
}

class S2Base_ZfValidateSupportPlugin extends Zend_Controller_Plugin_Abstract
{
    const VALIDATE_DIR = 'validate';
    const DEFAULT_KEY  = 'default';
    const ERRORS_KEY   = 's2base_validate_errors';
    const ERR_KEY      = 's2base_validate_error';
    const ALNUM_KEY  = 'alnum';
    const ALPHA_KEY  = 'alpha';
    const DATE_KEY   = 'date';
    const FLOAT_KEY  = 'float';
    const INT_KEY    = 'int';
    const IP_KEY     = 'ip';
    private static $VALIDATE_CLASSES = array(self::ALNUM_KEY  => 'Zend_Validate_Alnum',
                                             self::ALPHA_KEY  => 'Zend_Validate_Alpha',
                                             self::DATE_KEY   => 'Zend_Validate_Date',
                                             self::FLOAT_KEY  => 'Zend_Validate_Float',
                                             self::INT_KEY    => 'Zend_Validate_Int',
                                             self::IP_KEY     => 'Zend_Validate_Ip');
    private $validators = array();
    private $validateFactories = array();
    public static function hasError(Zend_Controller_Request_Abstract $request, $paramName = null) {
        if ($request->has(self::ERR_KEY)) {
            if ($paramName === null) {
                return $request->getParam(self::ERR_KEY);
            } else {
                $errors = $request->getParam(self::ERRORS_KEY);
                return isset($errors[$paramName]);
            }
        }
        return false;
    }
    public static function getErrors(Zend_Controller_Request_Abstract $request, $paramName = null) {
        if ($request->has(self::ERRORS_KEY)) {
            if ($paramName === null) {
                return $request->getParam(self::ERRORS_KEY);
            } else {
                $errors = $request->getParam(self::ERRORS_KEY);
                if (isset($errors[$paramName])) {
                    return $errors[$paramName];
                }
            }
        }
        return array();
    }
    public function addValidateFactory(S2Base_ZfValidateFactory $validateFactory, $key = null) {
        if ($key === null) {
            $key = $validateFactory->getId();
        }
        if (isset(self::$VALIDATE_CLASSES[$key]) or
            isset($this->validateFactories[$key])) {
            throw new S2Base_ZfException("can not add " . get_class($validateFactory) . ". key [$key] exists.");
        }
        $this->validateFactories[$key] = $validateFactory;
        return $this;
    }
    public function dispatchLoopStartup(Zend_Controller_Request_Abstract $request) {
        $moduleName = $request->getModuleName();
        $controllerName = $request->getControllerName();
        $actionName = $request->getActionName();
        $validateIni = S2BASE_PHP5_ROOT
                     . "/app/modules/$moduleName/$controllerName/"
                     . self::VALIDATE_DIR
                     . "/$actionName.ini";
        if (!is_file($validateIni)) {
            return;
        }
        $validateConfig = new Zend_Config_Ini($validateIni, null);
        $forceBreak = false;
        $errors = array();
        while ($validateConfig->valid()) {
            $paramName = strtolower($validateConfig->key());
            if ($paramName == self::DEFAULT_KEY) {
                $validateConfig->next();
                continue;
            }
            $paramConfig = $validateConfig->current();
            $vals = preg_split('/,/', $paramConfig->validate, -1, PREG_SPLIT_NO_EMPTY);
            foreach ($vals as $valKey) {
                $valKey = strtolower(trim($valKey));
                $validator = $this->getValidatorInstance($valKey, $paramConfig, $paramName);
                $paramValue = $request->getParam($paramName);
                if (!$validator->isValid($paramValue)){
                    if($paramConfig->exception != null) {
                        throw new S2Base_ZfException($paramConfig->exception);
                    }
                    if ($paramConfig->$valKey !== null and $paramConfig->$valKey->msg !== null) {
                        $msg = $paramConfig->$valKey->msg;
                    } else { 
                        $msg = implode('. ', $validator->getMessages());
                    }
                    $errors[$paramName] = array('value'   => $paramValue,
                                                'msg'     => $msg,
                                                'pre_mod' => $moduleName,
                                                'pre_ctl' => $controllerName,
                                                'pre_act' => $actionName);
                    if($paramConfig->module != null) {
                        $request->setModuleName($paramConfig->module);
                    }
                    if($paramConfig->controller != null) {
                        $request->setControllerName($paramConfig->controller);
                    }
                    if($paramConfig->action != null) {
                        $request->setActionName($paramConfig->action);
                    }
                    if (strtolower($paramConfig->break) == 'true') {
                        $forceBreak = true;
                        break;
                    }
                }
            }
            if ($forceBreak) {
                break;
            }
            $validateConfig->next();
        }
        if (count($errors) > 0) {
            $request->setParam(self::ERR_KEY, true);
            $request->setParam(self::ERRORS_KEY, $errors);
        } else {
            $request->setParam(self::ERR_KEY, false);
        }
    }
    private function getValidatorInstance($valKey, $paramConfig, $paramName) {
        $validator = null;
        switch($valKey) {
            case self::ALNUM_KEY:
            case self::ALPHA_KEY:
            case self::DATE_KEY:
            case self::FLOAT_KEY:
            case self::INT_KEY:
            case self::IP_KEY:
                if (isset($this->validators[$valKey])) {
                    $validator = $this->validators[$valKey];
                } else {
                    $valClassName = self::$VALIDATE_CLASSES[$valKey];
                    Zend_Loader::loadClass($valClassName);
                    $validator = new $valClassName();
                    $this->validators[$valKey] = $validator;
                }
                break;
             default:
                if (isset($this->validateFactories[$valKey])) {
                    $validator =  $this->validateFactories[$valKey]->getInstance($paramName, $paramConfig);
                    if (! $validator instanceof Zend_Validate_Interface) {
                        throw new S2Base_ZfException("$valKey validate not implements Zend_Validate_Interface");
                    }
                } else {
                    throw new S2Base_ZfException("unsupported validate [$valKey]");
                }
                break;
        }
        return $validator;
    }
}

class S2Base_ZfSmartyViewRenderer extends Zend_Controller_Action_Helper_Abstract {
    public static $config   = array();
    private static $errors   = array();
    private $layout     = null;
    private $template   = null;
    private $_noRender  = false;
    public function setTemplateDir($templateDir) {
        $this->templateDir = $templateDir;
    }
    public function getTemplateDir() {
        return $this->templateDir;
    }
    public function setTemplate($template) {
        $this->template = $template;
    }
    public function getTemplate() {
        return $this->template;
    }
    public function setTpl($tpl) {
        $this->setTemplate($tpl);
    }
    public function getTpl() {
        return $this->getTemplate();
    }
    public function setLayout($layout){
        $this->layout = $layout;
    }
    public function getLayout(){
        return $this->layout;
    }
    public function putError($key,$val){
        self::$errors[$key] = $val;
    }
    public function getError($key){
        if(isset(self::$errors[$key])){
            return self::$errors[$key];
        }
        return null;
    }
    public function getErrors(){
        return self::$errors;
    }
    public function setNoRender($flag = true)
    {
        $this->_noRender = ($flag) ? true : false;
        return $this;
    }
    public function getNoRender()
    {
        return $this->_noRender;
    }
    public function getName() {
        return 'ViewRenderer';
    }
    public function __construct() {
        $this->view = new Smarty();
        foreach(self::$config as $key=>$val){
            $this->view->$key = $val;
        }
        if(defined('S2BASE_PHP5_LAYOUT')){
            $this->layout = S2BASE_PHP5_LAYOUT;
        }
        $this->templateDir = S2BASE_PHP5_ROOT . '/app/modules';
    }
    public function init()
    {
        if (Zend_Controller_Front::getInstance()->getParam('noViewRenderer')) {
            return;
        }
        if ((null !== $this->_actionController) && (null === $this->_actionController->view)) {
            $this->_actionController->view = $this->view;
        }
    }
    public function postDispatch()
    {
        if (Zend_Controller_Front::getInstance()->getParam('noViewRenderer')) {
            return;
        }
        if (!$this->_noRender 
            && (null !== $this->_actionController)
            && $this->getRequest()->isDispatched()
            && !$this->getResponse()->isRedirect())
        {
            $this->render($this->getRequest()->getActionName());
        }
    }
    public function render($script) {
        $this->putError('validate', S2Base_ZfValidateSupportPlugin::getErrors($this->getRequest()));
        $moduleName = $this->getRequest()->getModuleName();
        $this->templateDir = $this->templateDir;
        $this->view->assign('request',$this->getRequest());
        $this->view->assign('errors',self::$errors);
        $this->view->assign('module', $moduleName);
        $this->view->assign('controller', $this->getRequest()->getControllerName());
        $this->view->assign('action', $this->getRequest()->getActionName());
        $this->view->assign('base_url', $this->getRequest()->getBaseUrl());
        $mod_url = $moduleName === S2BASE_PHP5_ZF_DEFAULT_MODULE ?
                   $this->getRequest()->getBaseUrl() :
                   $this->getRequest()->getBaseUrl() . '/' . $moduleName;
        $ctl_url = $mod_url . '/' . $this->getRequest()->getControllerName();
        $act_url = $ctl_url . '/' . $this->getRequest()->getActionName();
        $this->view->assign('mod_url', $mod_url);
        $this->view->assign('ctl_url', $ctl_url);
        $this->view->assign('act_url', $act_url);
        $ctlViewDir = $moduleName
                    . DIRECTORY_SEPARATOR
                    . $this->getRequest()->getControllerName()
                    . DIRECTORY_SEPARATOR
                    . 'view';
        $this->view->assign('ctl_view_dir', $this->templateDir . DIRECTORY_SEPARATOR . $ctlViewDir);
        $this->view->assign('commons_view_dir', S2BASE_PHP5_ROOT . '/app/commons/view');
        if ($this->template === null) {
            if (!preg_match('/\.' . S2BASE_PHP5_ZF_TPL_SUFFIX . '$/', $script)) {
                $script .= '.' . S2BASE_PHP5_ZF_TPL_SUFFIX;
            }
            $viewFile = $ctlViewDir . DIRECTORY_SEPARATOR . $script;
            if (!file_exists($this->templateDir . DIRECTORY_SEPARATOR . $viewFile)) {
                throw new S2Base_ZfException('template file not found. [' . 
                     $this->templateDir . DIRECTORY_SEPARATOR . $viewFile . ']');
            }
        } else {
            if (preg_match('/^file:/',$this->template)){
                $viewFile = $this->template;
            } else {
                if (!preg_match('/\.' . S2BASE_PHP5_ZF_TPL_SUFFIX . '$/', $this->template)) {
                    $this->template .= '.' . S2BASE_PHP5_ZF_TPL_SUFFIX;
                }
                $viewFile = $ctlViewDir . DIRECTORY_SEPARATOR . $this->template;
                if (!file_exists($this->templateDir . DIRECTORY_SEPARATOR . $viewFile)) {
                    throw new S2Base_ZfException('template file not found. [' . 
                         $this->templateDir . DIRECTORY_SEPARATOR . $viewFile . ']');
                }
            }
        }
        $this->view->template_dir = $this->templateDir;
        if($this->layout === null){
            $this->getResponse()->appendBody($this->view->fetch($viewFile));
        }else{
            $this->assign('content_for_layout', $viewFile);
            $this->getResponse()->appendBody($this->view->fetch($this->layout));
        }
        $this->setNoRender();
    }
    public function direct($template)
    {
        $this->setTemplate($template);
    }
}

class S2Base_ZfDb {
    private function __construct(){}
    public static function factory($pdoDicon) {
        $container = S2ContainerFactory::create($pdoDicon);
        $cd = $container->getComponentDef('dataSource');
        $username = null;
        $password = null;
        if ($cd->hasPropertyDef('user')) {
            $username = $cd->getPropertyDef('user')->getValue();
        }
        if ($cd->hasPropertyDef('password')) {
            $password = $cd->getPropertyDef('password')->getValue();
        }
        $dsn = $cd->getPropertyDef('dsn')->getValue();
        list($pdoType, $pdoParams)= preg_split('/:/', $dsn, 2);
        $pdoType = 'PDO_' . $pdoType;
        $params = array('username' => $username, 'password' => $password);
        if (preg_match('/;/', $pdoParams)) {
            $items = preg_split('/;/', $pdoParams, -1, PREG_SPLIT_NO_EMPTY);
            foreach ($items as $item) {
                list($key, $val)= preg_split('/=/', $item, 0);
                $params[trim($key)] = trim($val);
            }
        } else {
            $params['dbname'] = $pdoParams;
        }
        return Zend_Db::factory($pdoType, $params);
    }
    public static function setDefaultPdoAdapter($pdoDicon = PDO_DICON) {
        Zend_Db_Table::setDefaultAdapter(self::factory($pdoDicon));
    }
}

?>
